@@ -10,8 +10,10 @@ logger = logging.getLogger(__name__)
 
 class ManagerClientThread(threading.Thread):
 
-    def __init__(self, host, port, generate_message, on_message):
+    def __init__(self, host, port, generate_message, on_message,
+                 socket_lock=None):
         super(ManagerClientThread, self).__init__()
+        self.lock = socket_lock
         self.daemon = True
         self.host = host
         self.port = port
@@ -22,12 +24,20 @@ class ManagerClientThread(threading.Thread):
     def run(self):
         """Connect to manager and read/write messages from/to socket."""
         logger.debug("Running manager client thread")
+        if self.lock:
+            self.lock.acquire()
+
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         try:
             logger.debug("Connection to manager")
             sock.connect((self.host, self.port))
             logger.debug("Connected to manager")
+            if self.lock:
+                self.lock.release()
+
             message_loop(sock, self.generate_message, self.on_message)
         finally:
             logger.debug("Closing socket")
             sock.close()
+            if self.lock and self.lock.locked():
+                self.lock.release()
