@@ -8,25 +8,25 @@ import logging
 import resource
 import itertools
 import threading
-from time import time, sleep
 from collections import namedtuple
 
 from setproctitle import setproctitle
 
-from kuyruk import __version__
+from kuyruk import __version__, helpers
 from kuyruk.process import KuyrukProcess
 
+
 logger = logging.getLogger(__name__)
 
 
 class Master(KuyrukProcess):
-    """Spawns multiple worker and supervises them by reading the list of queues
+    """Spawn multiple workers and supervise them by reading the list of queues
     from configuration.
 
     """
     def __init__(self, config):
         super(Master, self).__init__(config)
-        self.workers = []
+        self.workers = {}
         self.lock = threading.Lock()
 
     def run(self):
@@ -38,15 +38,15 @@ class Master(KuyrukProcess):
         logger.debug('End run master')
 
     def start_workers(self):
-        """Starts a new worker for each queue."""
+        """Start a new worker for each queue."""
         queues = self.get_queues()
         queues = parse_queues_str(queues)
         logger.info('Starting to work on queues: %s', queues)
         for queue in queues:
-            self.spawn_new_worker(queue)
+            self.create_new_worker(queue)
 
     def get_queues(self):
-        """Returns queues string by reading from configuration."""
+        """Return queues string by reading from configuration."""
         if isinstance(self.config.QUEUES, basestring):
             return self.config.QUEUES
 
@@ -60,13 +60,13 @@ class Master(KuyrukProcess):
             return self.config.DEFAULT_QUEUES
 
     def shutdown_workers(self, kill=False):
-        """Sends shutdown signal to all workers."""
+        """Send shutdown signal to all workers."""
         if kill:
             fn, sig = os.killpg, signal.SIGKILL
         else:
             fn, sig = os.kill, signal.SIGTERM
 
-        for worker in self.workers:
+        for worker in self.workers.itervalues():
             try:
                 logger.info("Sending signal %s to PID %s", sig, worker.pid)
                 fn(worker.pid, sig)
@@ -78,44 +78,34 @@ class Master(KuyrukProcess):
                     raise
 
     def wait_for_workers(self):
-        """Loops until any of the self.workers is alive.
+        """Loop until any of the self.workers is alive.
         If a worker is dead and Kuyruk is running state, spawns a new worker.
 
         """
-        start = time()
-        any_alive = True
-        while any_alive:
-            any_alive = False
-            for worker in list(self.workers):
-                if worker.is_alive():
-                    any_alive = True
-                elif not self.shutdown_pending.is_set():
-                    worker.kill_pg()
-                    self.respawn_worker(worker)
-                    any_alive = True
-                else:
-                    self.workers.remove(worker)
-
-            if any_alive:
-                logger.debug("Waiting for workers... "
-                             "%i seconds passed" % (time() - start))
-                sleep(1)
+        retry_wait = helpers.retry_on_eintr(os.wait)
+        while self.workers:
+            pid, status = retry_wait()
+            # Worker is dead
+            worker = self.workers[pid]
+            worker.kill_pg()
+            del self.workers[pid]
+            if not self.shutdown_pending.is_set():
+                self.respawn_worker(worker)
 
     def respawn_worker(self, worker):
-        """Spawns a new process with parameters same as the old worker."""
+        """Spawn a new process with parameters same as the old worker."""
         logger.debug("Respawning worker %s", worker)
-        self.spawn_new_worker(worker.queue)
-        self.workers.remove(worker)
+        self.create_new_worker(worker.queue)
         logger.debug(self.workers)
 
-    def spawn_new_worker(self, queue):
+    def create_new_worker(self, queue):
         if self.shutdown_pending.is_set():
             logger.info("Shutdown is pending. Skipped spawning new worker.")
             return
         worker = WorkerProcess(self.kuyruk, queue)
         with self.lock:
             worker.start()
-        self.workers.append(worker)
+        self.workers[worker.pid] = worker
 
     def register_signals(self):
         super(Master, self).register_signals()
@@ -123,10 +113,6 @@ class Master(KuyrukProcess):
         signal.signal(signal.SIGQUIT, self.handle_sigquit)
         signal.signal(signal.SIGABRT, self.handle_sigabrt)
 
-        # Ignore SIGCHLD explicitly so the kernel to
-        # automatically reap child processes.
-        signal.signal(signal.SIGCHLD, signal.SIG_IGN)
-
     def handle_sigterm(self, signum, frame):
         logger.warning("Handling SIGTERM")
         self.warm_shutdown()
@@ -149,12 +135,12 @@ class Master(KuyrukProcess):
         self.shutdown_workers(kill=True)
 
     def abort(self):
-        """Exits immediately making workers orphan."""
+        """Exit immediately making workers orphan."""
         logger.warning("Aborting")
         os._exit(1)
 
     def get_stats(self):
-        """Generates stats to be sent to manager."""
+        """Generate stats to be sent to manager."""
         return {
             'type': 'master',
             'hostname': socket.gethostname(),
@@ -231,7 +217,7 @@ class WorkerProcess(object):
             return True
 
     def kill_pg(self):
-        """Kills the process with their children. Does not raise exception
+        """Kill the process with their children. Does not raise exception
         if the process is not alive.
 
         """
@@ -244,7 +230,7 @@ class WorkerProcess(object):
 
 
 def parse_queues_str(s):
-    """Parses command line queues string.
+    """Parse command line queues string.
 
     :param s: Command line or configuration queues string
     :return: list of queue names
